/**
 * This file contains the listener functions which can be assigned to the
 * coprus_analysis client. So that the incoming data/status informations will
 * be handled. There are several listeners listening for socket .io events.
 * Further below one javascript custom event listener is specified. This
 * listener listens for javascript custom events which are being dispatched by
 * the View (resultsList).
 */

// Listeners for socket io events

/**
 * Recieves a corpus analysis connected signal via socket.io.
 */
function recieveConnected(type, client) {
  client.socket.on(type, (response) => {
    /**
     * Check if request for session was OK.
     * If OK execute registered callbacks and notify View.
     */
    if (response.code === 200) {
      console.group('Connected!')
      console.info('corpus_analysis_init: Client recieving connected codes',
                   'codes via socket.on');
      console.info(`corpus_analysis_init: ${response.code} - ${response.msg}`);
      console.info('corpus_analysis_init: Initialization succeeded');
      console.info(response);
      client.notifyView('connected');
      console.groupEnd();
      // get meta data immediately
      client.getMetaData();
    } else {
      let errorText = `Error ${response.code} - ${response.msg}`;
      console.group('Connection failed!');
      console.error(`corpus_analysis_init: ${errorText}`);
      client.notifyView('client-failed', { msg: errorText }, 'error');
      console.groupEnd();
    }
  });
}

/**
 * Recieves meta data from the server via socket.io.
 */
function recieveMetaData(type, client) {
  client.socket.on(type, (response) => {
    /**
     * Check if request for session was OK.
     * If OK execute registered callbacks and notify View.
     */
    if (response.code === 200) {
      console.group('Client recieving meta data')
      console.info('corpus_analysis_meta_data: Client recieving meta data',
      'via socket.on');
      console.info(`corpus_analysis_meta_data: ${response.code} - ${response.msg}`);
      console.info(response);
      // executing the registered callbacks
      client.eventListeners[type].executeCallbacks([response.payload]);
      console.groupEnd();
    } else {
      let errorText = `Error ${response.payload.code} - ${response.payload.msg}`;
      console.group('Failed to recieve meta data.');
      console.error('corpus_analysis_meta_data: Client failed to recieve',
                    'meta data via socket.on');
      console.error(`corpus_analysis_meta_data: ${errorText}`);
      client.notifyView('client-failed', { msg: errorText }, 'error');
      console.groupEnd();
    }
  });
}

/**
 * Recieves the query process status before any actual results are being
 * transmitted. So it recieves error codes if a query failed or
 * was invalid etc.
 * Also prepares the result.jsList for the incoming data.
 */
function recieveQueryStatus(type, client) {
  /**
   * Check if request for session was OK.
   * If OK execute registered callbacks and notify View.
   */
  client.socket.on(type, (response) => {
    if (response.code === 200) {
      console.group('corpus_analysis_query: Client recieving query process',
      'status via socket.on');
      console.info(`corpus_analysis_query: ${response.code} - ${response.msg}`);
      console.info(response);
      // executing the registered callbacks
      client.eventListeners[type].executeCallbacks([response.payload]);
      console.groupEnd();
    } else {
      let errorText = `Error ${response.payload.code} - ${response.payload.msg}`;
      console.group('corpus_analysis_query: Client failed recieving',
      'query process status via socket.on');
      if (response.payload.code == 1281) {
        errorText += ' - Invalid Query';
        console.error(`corpus_analysis_query: ${errorText}`);
        client.notifyView('client-failed', { msg: errorText }, 'error', false);
      } else {
        console.error(`corpus_analysis_query: ${errorText}`);
        client.notifyView('client-failed', { msg: errorText }, 'error');
      }
      console.groupEnd();
    }
  });
}

/**
 * Recieves the query data from the request and handles it.
 */
function recieveQueryData(type, client) {
  /**
   * Check if request for session was OK.
   * If OK execute registered callbacks and notify View.
   */
  client.socket.on(type, (response) => {
    if (response.code === 200) {
      console.group('corpus_analysis_query_results: Recieveing query data')
      console.info('Client recieving query data via socket.on');
      console.info('Recieved chunk', response);
      /**
       * Execute registered callbacks and notify View.
       */
      client.eventListeners[type].executeCallbacks([response.payload]);
      console.info('Added chunk data to results.data.');
      console.groupEnd();
    } else {
      let errorText = `Error ${response.payload.code} - ${response.payload.msg}`;
      console.group('corpus_analysis_query_results: Client failed recieving',
                    'the results via socket.on');
      console.error(`corpus_analysis_query: ${errorText}`);
      client.notifyView('client-failed', { msg: errorText }, 'error');
      console.groupEnd();
    }
  });
}

/**
 * Recieves the data requested by the create Results or sub results button
 */
function recieveResultsData(type, client) {
  client.socket.on(type, (response) => {
    /**
     * Check if request for session was OK.
     * If OK execute registered callbacks and notify View.
     */
    if (response.code === 200) {
      console.group('Client recieving results data')
      console.info('corpus_analysis_inspect_match: Client recieving results data',
      'via socket.on');
      console.info(`corpus_analysis_inspect_match: ${response.code} - ${response.msg}`);
      console.info(response);
      // executing the registered callbacks
      client.eventListeners[type].executeCallbacks([response.payload,
                                                    response.type]);
      console.groupEnd();
    } else {
      let errorText = `Error ${response.payload.code} - ${response.payload.msg}`;
      console.group('Failed to recieve results data.');
      console.error('corpus_analysis_inspect_match: Client failed to recieve',
                    'results data via socket.on');
      console.error(`corpus_analysis_inspect_match: ${errorText}`);
      client.notifyView('client-failed', { msg: errorText }, 'error');
      console.groupEnd();
    }
  });
}

/*
 * This is the javascript custom event listener, listening for events
 * dispatched by the View.
 */
function recieveViewNotification(type, client) {
  document.addEventListener(type, (event) => {
    let caseIdentifier = event.detail.caseIdentifier;
    switch(caseIdentifier) {
      case 'get-results':
        console.info('Client getting full results for export.');
        // execute callback or functions
        client.eventListeners[type].executeCallback([event.detail.resultsType,
                                                     event.detail.dataIndexes],
                                                     caseIdentifier);
        break
        default:
          console.error('Recieved unkown notification case identifier from View');
          // do something to not crash the analysis session?
          // maybe unnecessary?
    }
  });
}

// export listeners from this module
export {
  recieveConnected,
  recieveMetaData,
  recieveQueryStatus,
  recieveQueryData,
  recieveViewNotification,
  recieveResultsData,
};